using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.IO;

namespace Mandelbrot
{
	class ComplexNumber
	{
		private double r; // Reálná část komplexního čísla.
		private double i; // Imaginární část komplexního čísla.
		
		public double R
		{
			get
			{
				return this.r;
			}
			set
			{
				this.r = value;
			}
		}
		
		public double I
		{
			get
			{
				return this.i;
			}
			set
			{
				this.i = value;
			}
		}
		
		public ComplexNumber ()
		{
			this.R = 0;
			this.I = 0;
		}
		
		public ComplexNumber (double r, double i)
		{
			this.R = r;
			this.I = i;
		}
		
		public ComplexNumber (ComplexNumber num)
		{
			this.R = num.R;
			this.I = num.I;
		}
		
		// Vrátí komplexně sdružené číslo k tomuto číslu.
		public static ComplexNumber ComplexConjugate (ComplexNumber num)
		{
			return new ComplexNumber (num.R, -num.I);
		}
		
		// Tohle je unární - (jinak řečeno, vytvoří opačné komplexní číslo).
		public static ComplexNumber operator - (ComplexNumber num)
		{
			return new ComplexNumber (-num.R, -num.I);
		}
		
		// Tohle je binární -.
		public static ComplexNumber operator - (ComplexNumber a, ComplexNumber b)
		{
			return new ComplexNumber (a.R - b.R, a.I - b.I);
		}
		
		// Tohle je binární +.
		public static ComplexNumber operator + (ComplexNumber a, ComplexNumber b)
		{
			return new ComplexNumber (a.R + b.R, a.I + b.I);
		}
		
		// Tohle je binární *.
		public static ComplexNumber operator * (ComplexNumber a, ComplexNumber b)
		{
			return new ComplexNumber (a.R * b.R - a.I * b.I, a.R * b.I + a.I * b.R);
		}
		
		// Tohle je binární /.
		public static ComplexNumber operator / (ComplexNumber a, ComplexNumber b)
		{
			// Dělení probíhá pomocí triku, kdy celký zlomek vynásobím
			// číslem komplexně sdruženým k b. Tím ve jmenovateli dostanu
			// součet dvou reálných čísel a v čitateli získám součin dvou
			// komplexních čísel, což už je možné spočítat.
			double denomination = b.R * b.R + b.I * b.I;
			ComplexNumber numerator = a * ComplexNumber.ComplexConjugate (b);
			return new ComplexNumber(numerator.R / denomination,
				numerator.I / denomination);
		}
		
		// Tohle je (zatím) celočíselná mocnina.
		public static ComplexNumber Pow (ComplexNumber num, int exponent)
		{
			if (exponent < 2)
			{
				return new ComplexNumber (num);
			}
			return new ComplexNumber (num * ComplexNumber.Pow (num, exponent - 1));
		}
		
		// Tohle je absolutní hodnota.
		public static double Abs (ComplexNumber num)
		{
			return Math.Sqrt (num.R * num.R + num.I * num.I);
		}
		
		// Přepíšeme si metodu toString.
		public override string ToString ()
		{
			//return string.Format ("[ComplexNumber: R={0}, I={1}]", R, I);
			return string.Format ("({0} + {1}i)", this.R, this.I);
		}
	}
	
	class RGB
	{
		private int r = 0;
		private int g = 0;
		private int b = 0;
		
		public int R
		{
			get
			{
				return this.r;
			}
			set
			{
				if ((value >= 0) && (value <= 255))
				{
					this.r = value;
				}
			}
		}
		
		public int G
		{
			get
			{
				return this.g;
			}
			set
			{
				if ((value >= 0) && (value <= 255))
				{
					this.g = value;
				}
			}
		}
		
		public int B
		{
			get
			{
				return this.b;
			}
			set
			{
				if ((value >= 0) && (value <= 255))
				{
					this.b = value;
				}
			}
		}
		
		public RGB (int p_r, int p_g, int p_b)
		{
			this.R = p_r;
			this.G = p_g;
			this.B = p_b;
		}
	}
	
	class Palette
	{
		private SortedList<int, RGB> rgb;
		private bool special = false;
		private string file = "";
		private string[] specialStr = new string[] { MandelbrotSet.SPECCOL,
		MandelbrotSet.SPECCOL, MandelbrotSet.SPECCOL };
		
		public bool Special
		{
			get
			{
				return this.special;
			}
		}
		
		public Palette (string p_file, bool p_special)
		{
			this.file = p_file;
			this.special = p_special;
		}
		
		private RGB parseLine (string line, int numberOfLine)
		{
			string[] values = line.Split (' ');
			int[] color = new int[3];
			
			try
			{
				color[0] = int.Parse (values[0]);
				color[1] = int.Parse (values[1]);
				color[2] = int.Parse (values[2]);
			}
			catch
			{
				color[0] = 0;
				color[1] = 0;
				color[2] = 0;
				Console.WriteLine ("Parsing error on line {0}. Setting this" +
					" RGB value to (0, 0, 0).", numberOfLine);
			}
			return new RGB (color[0], color[1], color[2]);
		}
		
		public void LoadFromFile ()
		{
			if (File.Exists (this.file))
			{
				try
				{
					this.rgb = new SortedList<int, RGB> ();
					using (StreamReader sr = File.OpenText (this.file))
					{
						int num = 0;
						while (!sr.EndOfStream)
						{
							num++;
							string line = sr.ReadLine ();
							this.rgb.Add (num - 1, this.parseLine (line, num));
						}
					}
				}
				catch (Exception e)
				{
					Console.WriteLine ("\tError!");
					Console.WriteLine (e.Message);
					Console.WriteLine ("Own palette will not be used.");
					this.special = true;
				}
			}
			else
			{
				Console.WriteLine ("\tError!");
				Console.WriteLine ("File \"{0}\" does not exists.", this.file);
				Console.WriteLine ("Own palette will not be used.");
				this.special = true;
			}
		}
		
		public void SetSpecial (string r, string g, string b)
		{
			this.specialStr[0] = r;
			this.specialStr[1] = g;
			this.specialStr[2] = b;
		}
		
		public Color GetColor (int i)
		{
			if (this.special)
			{
				int r, g, b;
				
				if (this.specialStr[0] == "+")
				{
					r = i;
				}
				else if (this.specialStr[0] == "-")
				{
					r = 255 - i;
				}
				else
				{
					r = int.Parse (this.specialStr[0]);
				}
				
				if (this.specialStr[1] == "+")
				{
					g = i;
				}
				else if (this.specialStr[1] == "-")
				{
					g = 255 - i;
				}
				else
				{
					g = int.Parse (this.specialStr[1]);
				}
				
				if (this.specialStr[2] == "+")
				{
					b = i;
				}
				else if (this.specialStr[2] == "-")
				{
					b = 255 - i;
				}
				else
				{
					b = int.Parse (this.specialStr[2]);
				}
				return Color.FromArgb (r, g, b);
			}
			else
			{
				int c = i;
				if (c >= this.rgb.Count)
				{
					while ((c -= this.rgb.Count) >= this.rgb.Count)
						;
				}
				RGB t_rgb = new RGB(0, 0, 0);
				if (this.rgb.TryGetValue (c, out t_rgb))
				{
					return Color.FromArgb (t_rgb.R, t_rgb.G, t_rgb.B);
				}
				else
				{
					return Color.Black;
				}	
			}
		}
	}
	
	class MandelbrotSet
	{
		// Konstanty.
		public const int MAXITER = 255;
		public const double WIDTHX = 2.0;
		public const double WIDTHY = 2.0;
		public const int EXPONENT = 2;
		public const double SCALE = 1.0;
		public const double POSR = 0.0;
		public const double POSI = 0.0;
		public const int WIDTH = 512;
		public const int HEIGHT = 512;
		public const string OUTPUTFILE = "out.png";
		public const bool OWNPALETTE = false;
		public const string SPECCOL = "+";
		
		private int maxIter = MAXITER; // Maximální počet iterací.
		
		// Výřez obrazce.
		private double minX;
		private double maxX;
		private double minY;
		private double maxY;
		
		// Absolutní hranice obrazce v komplexní rovině.
		private double widthX = WIDTHX;
		private double widthY = WIDTHY;
		
		private int exponent = EXPONENT; // Exponent, použitý pro výpočet.
		
		private double scale = SCALE; // Měřítko.
		private ComplexNumber pos = new ComplexNumber(POSR, POSI); // Posun obrázku.
		
		private int width = WIDTH; // Výchozí šířka obrázku.
		private int height = HEIGHT; // Výchozí výška obrázku.
		
		private Palette palette; // Paleta.
		
		private Bitmap bmp;
		
		public MandelbrotSet (int p_width, int p_height, double p_widthX,
			double p_widthY, double p_scale, ComplexNumber p_pos, int p_exponent,
			int p_maxIter, Palette p_palette)
		{
			this.width = p_width;
			this.height = p_height;
			// Nesmí být větší než 2.0.
			this.widthX = (p_widthX < this.widthX) ? p_widthX : this.widthX;
			this.widthY = (p_widthY < this.widthY) ? p_widthY : this.widthY;
			this.scale = p_scale;
			if ((p_pos.R >= -this.widthX) && (p_pos.R <= this.widthX) &&
				(p_pos.I >= -this.widthY) && (p_pos.I <= this.widthY))
			{
				this.pos = p_pos;
			}
			this.exponent = (p_exponent > this.exponent) ? p_exponent : this.exponent;
			this.maxIter = (p_maxIter < this.maxIter) ? p_maxIter : this.maxIter;
			this.palette = p_palette;
			this.ComputeCorners ();
			bmp = new Bitmap (this.width, this.height, PixelFormat.Format24bppRgb);
		}
		
		// Spočte hranice výřezu a měřítko.
		private void ComputeCorners ()
		{
			this.minX = this.pos.R - this.widthX * this.scale;
			this.minY = this.pos.I - this.widthY * this.scale;
			this.maxX = this.pos.R + this.widthX * this.scale;
			this.maxY = this.pos.I + this.widthY * this.scale;
		}
		
		// Vykreslí Mandelbrotovu množinu.
		public void ComputeMandelbrot ()
		{
			int i;
			int x, y;
			ComplexNumber c0 = new ComplexNumber (this.minX, this.minY);
			ComplexNumber c = new ComplexNumber ();
			ComplexNumber z = new ComplexNumber ();
			
			for (y = 0; y < this.height; y++)
			{
				c0.R = this.minX;
				for (x = 0; x < this.width; x++)
				{
					c.R = c0.R;
					c.I = c0.I;
					z.R = 0;
					z.I = 0;
					for (i = 0; i < this.maxIter; i++)
					{
						z = ComplexNumber.Pow (z, this.exponent) + c;
						if (ComplexNumber.Abs (z) > 2.0)
						{
							break;
						}
					}
					this.bmp.SetPixel (x, y, this.palette.GetColor (i));
					c0.R += (this.maxX - this.minX) / this.width;
				}
				c0.I += (this.maxY - this.minY) / this.height;
			}
		}
		
		public void SaveImage (string outputFile)
		{
			this.bmp.Save (outputFile, ImageFormat.Png);
		}
	}
	
	class Arguments
	{
		private string[] args;
		private string[] validArgs;
		private Dictionary<string, string> argsDict =
			new Dictionary<string, string>();
		
		public Dictionary<string, string> ArgsDictionary
		{
			get
			{
				return this.argsDict;
			}
		}
		
		public Arguments (string[] p_args, string[] p_validArgs)
		{
			this.args = p_args;
			this.validArgs = p_validArgs;
		}
		
		public int GetCount ()
		{
			// Používám počet prvků args, protože toto pole je určitě naplněno
			// již při inicializaci této třídy.
			return this.args.Length;
		}
		
		public bool Parse ()
		{
			foreach (string s in this.args)
			{
				// Přidáme veškeré parametry a jejich hodnoty do slovníku.
				string[] c = s.Split ('=');
				this.argsDict.Add (c[0].Replace ("-", ""), (c.Length < 2) ? "" : c[1]);
			}
			foreach (KeyValuePair<string, string> pair in this.argsDict)
			{
				// Kontrola neznámých parametrů.
				if (!((IList<string>)this.validArgs).Contains (pair.Key))
				{
					// To s tím rozhraním je dobrý trik, to bych si měl
					// pamatovat, třeba už jen proto, že arrays neobsahují
					// zde volanou metodu Contains :D
					Console.WriteLine ("\tError!");
					Console.WriteLine ("Unknown argument: \"{0}\" (with value: \"{1}\")",
						pair.Key, pair.Value);
					return false;
				}
			}
			return true;
		}
	}
	
	static class MyDoubleExtension
	{
		public static bool AlmostEqual (this double d1, double d2, double precision)
		{
			// To s tím this je zajímavé, v hlavním kódu se to potom použije
			// takto: double1.AlmostEqual (d2, precision); Super :)
			return (Math.Abs (d1 - d2) <= precision);
		}
	}
	
	class MainClass
	{
		public static void Main (string[] args)
		{
			string[] validArgs = new string[] { "iterations", "i", "bmWidth",
			"w", "bmHeight", "h", "maxAbsReal", "R", "maxAbsImag", "I",
			"scale", "s", "exponent", "e", "output", "o", "overwrite", "v",
			"posX", "x",
			"posY", "y", "r", "g", "b", "palette", "p" };
			Arguments a = new Arguments (args, validArgs);
			
			Console.WriteLine ();
			
			if (a.GetCount () == 0)
			{
				Console.WriteLine (" Mandelbrot Set Drawer app by Filip Loužecký\n");
				Console.WriteLine (" Usage:\t[--iterations=<value> | -i=<value>]");
				Console.WriteLine (" \t[--bmWidth=<value> | -w=<value>] " +
					"[--bmHeight=<value> | -h=<value>]");
				Console.WriteLine (" \t[--posX=<value> | -x=<value>] " +
					"[--posY=<value> | -y=<value>]");
				Console.WriteLine (" \t[--maxAbsReal=<value> | -R=<value>] " +
					"[--maxAbsImag=<value> | -I=<value>]");
				Console.WriteLine (" \t[--scale=<value> | -s=<value>]");
				Console.WriteLine (" \t[--exponent=<value> | -e=<value>]");
				Console.WriteLine (" \t[--output=<File> | -o=<File>]");
				Console.WriteLine (" \t[-r=<value>] [-g=<value>] [-b=<value>]");
				Console.WriteLine (" \t[--palette=<File> | -p=<File>]");
				Console.WriteLine (" \t[--overwrite | -v]");
				Console.WriteLine ("\n Arguments are:");
				Console.WriteLine ("  --iterations | -i\tThe count of iterations (def. {0}).", MandelbrotSet.MAXITER);
				Console.WriteLine ("  --bmWidth | -w\tThe width in pixels of the output bitmap (def. {0}).", MandelbrotSet.WIDTH);
				Console.WriteLine ("  --bmHeight | -h\tThe height in pixels of the output bitmap (def. {0}).", MandelbrotSet.HEIGHT);
				Console.WriteLine ("  --posX | -x\t\tThe x value of position of the center point (def. {0}).", MandelbrotSet.POSR);
				Console.WriteLine ("  --posY | -y\t\tThe y value of position of the center point (def. {0}).", MandelbrotSet.POSI);
				Console.WriteLine ("  --maxAbsReal | -R\tLeft and right border of the real part Gaussian plane\n \t\t\t(def {0}; the left will be -{1}, right {2}).",
					MandelbrotSet.WIDTHX, MandelbrotSet.WIDTHX, MandelbrotSet.WIDTHX);
				Console.WriteLine ("  --maxAbsImag | -I\tTop and bottom border of the imaginary part Gaussian\n \t\t\tplane (def {0}; the top will be -{1}, bottom {2}).",
					MandelbrotSet.WIDTHY, MandelbrotSet.WIDTHY, MandelbrotSet.WIDTHY);
				Console.WriteLine ("  --scale | -s\t\tThe scaling coefficient of the drawed Gaussian plane\n \t\t\t(def. {0} = {1}%).",
					MandelbrotSet.SCALE, Math.Round (MandelbrotSet.SCALE * 100));
				Console.WriteLine ("  --exponent | -e\tThe exponent in formula: Z(next) = Z(prev.)^exp + C\n \t\t\t(def. {0}).",
					MandelbrotSet.EXPONENT);
				Console.WriteLine ("  --output | -o\t\tThe path and name of the output image (def. {0}).", MandelbrotSet.OUTPUTFILE);
				Console.WriteLine ("  -r\t\t\tThe value of red value. Can be 0..255, - (255 - iter.)\n \t\t\tor + (0 + iter.), (def. {0}).",
					MandelbrotSet.SPECCOL);
				Console.WriteLine ("  -g\t\t\tThe value of green value (def. {0}).",
					MandelbrotSet.SPECCOL);
				Console.WriteLine ("  -b\t\t\tThe value of blue value (def. {0}).",
					MandelbrotSet.SPECCOL);
				Console.WriteLine ("  --palette | -p\tThe path and name of the file with own palette." +
					"\n \t\t\tIf entered valid file, the arguments -r, -g and -b will\n \t\t\tbe ignored.");
				Console.WriteLine ("  --overwrite | -v\tOverwrites the output file if exists (def. disabled).");
				Console.WriteLine ("\n For drawing the Mandelbrot Set must be at least one argument entered.");
			}
			else
			{
				Console.Write ("Parsing arguments...\t\t");
				if (a.Parse ())
				{
					Console.WriteLine ("\t[ OK ]");
				}
				else
				{
					return;
					// Exit by se mi líbilo více :)
				}
				Console.Write ("Preparing values of arguments...");
				
				int width = MandelbrotSet.WIDTH;
				int height = MandelbrotSet.HEIGHT;
				int exponent = MandelbrotSet.EXPONENT;
				int maxIter = MandelbrotSet.MAXITER;
				double widthX = MandelbrotSet.WIDTHX;
				double widthY = MandelbrotSet.WIDTHY;
				double scale = MandelbrotSet.SCALE;
				ComplexNumber pos =
					new ComplexNumber (MandelbrotSet.POSR, MandelbrotSet.POSI);
				string outputFile = MandelbrotSet.OUTPUTFILE;
				bool ownPalette = MandelbrotSet.OWNPALETTE;
				Palette palette;
				string[] special = new string[] { MandelbrotSet.SPECCOL,
				MandelbrotSet.SPECCOL, MandelbrotSet.SPECCOL };
				string paletteFile = "";
				bool overwrite = false;
				
				foreach (KeyValuePair<string, string> pair in a.ArgsDictionary)
				{
					switch (pair.Key)
					{
					case "iterations":
					case "i":
						try
						{
							maxIter = int.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected integer value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if ((maxIter < 0) || (maxIter > 255))
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Integer value must be in interval 0..255.",
								pair.Key);
							return;
						}
						break;
					case "bmWidth":
					case "w":
						try
						{
							width = int.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected integer value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if (width < 1)
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Integer value must be greater then 0.",
								pair.Key);
							return;
						}
						break;
					case "bmHeight":
					case "h":
						try
						{
							height = int.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected integer value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if (height < 1)
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Integer value must be greater then 0.",
								pair.Key);
							return;
						}
						break;
					case "maxAbsReal":
					case "R":
						try
						{
							widthX = double.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected double value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if ((widthX < 0.0) || (widthX > 2.0))
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Double value must be in interval <0; 2>.",
								pair.Key);
							return;
						}
						break;
					case "maxAbsImag":
					case "I":
						try
						{
							widthY = double.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected double value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if ((widthY < 0.0) || (widthY > 2.0)) {
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Double value must be in interval <0; 2>.",
								pair.Key);
							return;
						}
						break;
					case "posX":
					case "x":
						try
						{
							pos.R = double.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected double value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						break;
					case "posY":
					case "y":
						try
						{
							pos.I = double.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected double value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						break;
					case "scale":
					case "s":
						try
						{
							scale = double.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected double value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if (((scale < 0.0) || (scale > 1.0)) ||
							(scale.AlmostEqual (0.0, 0.0000000001)))
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Double value must be in interval (0; 1>.",
								pair.Key);
							return;
						}
						break;
					case "exponent":
					case "e":
						try
						{
							exponent = int.Parse (pair.Value);
						}
						catch
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Expected integer value, but \"{1}\" obtained.",
								pair.Key, pair.Value);
							return;
						}
						if (exponent < 2)
						{
							Console.WriteLine ("\tError!");
							Console.WriteLine ("Argument \"{0}\": Integer value must be greater then 1.",
								pair.Key);
							return;
						}
						break;
					case "overwrite":
					case "v":
						overwrite = true;
						break;
					case "output":
					case "o":
						outputFile = pair.Value;
						break;
					case "r":
						special[0] = pair.Value;
						if ((pair.Value != "+") && (pair.Value != "-"))
						{
							try
							{
								int.Parse (special[0]);
							}
							catch
							{
								Console.WriteLine ("\tError!");
								Console.WriteLine ("Argument \"{0}\": Expected integer value, or - or +, but \"{1}\" obtained.",
									pair.Key, pair.Value);
								return;
							}
						}
						break;
					case "g":
						special[1] = pair.Value;
						if ((pair.Value != "+") && (pair.Value != "-"))
						{
							try
							{
								int.Parse (special[1]);
							}
							catch
							{
								Console.WriteLine ("\tError!");
								Console.WriteLine ("Argument \"{0}\": Expected integer value, or - or +, but \"{1}\" obtained.",
									pair.Key, pair.Value);
								return;
							}
						}
						break;
					case "b":
						special[2] = pair.Value;
						if ((pair.Value != "+") && (pair.Value != "-"))
						{
							try
							{
								int.Parse (special[2]);
							}
							catch
							{
								Console.WriteLine ("\tError!");
								Console.WriteLine ("Argument \"{0}\": Expected integer value, or - or +, but \"{1}\" obtained.",
									pair.Key, pair.Value);
								return;
							}
						}
						break;
					case "palette":
					case "p":
						paletteFile = pair.Value;
						ownPalette = true;
						break;
					}
				}
				if ((!overwrite) && (File.Exists (outputFile)))
				{
					Console.WriteLine ("\tError!");
					// Pokud se budou měnit validní parametry, mělo by se upravit
					// i toto!
					string tmp = a.ArgsDictionary.ContainsKey ("o") ? "o" : "output";
					Console.WriteLine ("Argument \"{0}\": Entered file (\"{1}\") already exists." +
						"\nUse the argument \"--overwrite\" or \"-v\" to overwrite it.",
						tmp, outputFile);
					return;
				}
				if ((pos.R < -widthX) || (pos.R > widthX)) {
					Console.WriteLine ("\tError!");
					// Pokud se budou měnit validní parametry, mělo by se upravit
					// i toto!
					string tmp = a.ArgsDictionary.ContainsKey ("x") ? "x" : "posX";
					Console.WriteLine ("Argument \"{0}\": Double value must be in interval <{1}, {2}>.",
						tmp, -widthX, widthX);
					return;
				}
				if ((pos.I < -widthY) || (pos.I > widthY)) {
					Console.WriteLine ("\tError!");
					// Pokud se budou měnit validní parametry, mělo by se upravit
					// i toto!
					string tmp = a.ArgsDictionary.ContainsKey ("y") ? "y" : "posY";
					Console.WriteLine ("Argument \"{0}\": Double value must be in interval <{1}, {2}>.",
						tmp, -widthY, widthY);
					return;
				}
				Console.WriteLine ("\t[ OK ]");
				
				Console.Write ("Preparing palette...\t\t");
				
				if (ownPalette)
				{
					palette = new Palette (paletteFile, false);
					palette.LoadFromFile ();
					if (!palette.Special)
					{
						Console.WriteLine ("\t[ OK ]");
					}
				}
				else
				{
					palette = new Palette (paletteFile, true);
					Console.WriteLine ("\t[ OK ]");
				}
				palette.SetSpecial (special[0], special[1], special[2]);
				
				Console.WriteLine ("\n Settings:");
				Console.WriteLine ("  Bitmap width:\t\t\t{0}", width);
				Console.WriteLine ("  Bitmap height:\t\t{0}", height);
				Console.WriteLine ("  Max. count of iterations:\t{0}", maxIter);
				Console.WriteLine ("  Gaussian plane:\t\tR: {0} -- {1}" +
					"; I: {2} -- {3}", -widthX, widthX, -widthY, widthY);
				Console.WriteLine ("  Central point:\t\t[{0}; {1}]",
					pos.R, pos.I);
				Console.WriteLine ("  Scale:\t\t\t{0}", scale);
				Console.WriteLine ("  Exponent:\t\t\t{0}", exponent);
				Console.WriteLine ("  Output file:\t\t\t{0}", outputFile);
				if (palette.Special)
				{
					Console.WriteLine ("  Own palette:\t\t\tNo");
					Console.WriteLine ("  Color:\t\t\tRed value  : {0}", special[0]);
					Console.WriteLine ("        \t\t\tGreen value: {0}", special[1]);
					Console.WriteLine ("        \t\t\tBlue value : {0}", special[2]);
				}
				else
				{
					Console.WriteLine ("  Own palette:\t\t\tYes");
					Console.WriteLine ("  Palette file:\t\t\t{0}", paletteFile);
				}
				Console.WriteLine ();
				
				Console.Write ("Drawing Mandelbrot Set...\t");
				
				MandelbrotSet m = new MandelbrotSet (width, height, widthX, widthY,
					scale, pos, exponent, maxIter, palette);
				
				try
				{
					m.ComputeMandelbrot ();
				}
				catch (Exception e)
				{
					Console.WriteLine ("\tError!");
					Console.WriteLine (e.Message);
					return;
				}
				
				Console.WriteLine ("\t[ OK ]");
				Console.Write ("Writing the image to the file...",
					outputFile);
				
				try
				{
					m.SaveImage (outputFile);
				}
				catch (Exception e)
				{
					Console.WriteLine ("\tError!");
					Console.WriteLine (e.Message);
					return;
				}
				Console.WriteLine ("\t[ OK ]");
				Console.WriteLine ("\nDrawing of the Mandelbrot Set is complete! :)\n");
			}
		}
	}
}

